@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2020 gnome-mpv
+ * Copyright (c) 2014-2022 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -23,12 +23,16 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#define gtk_widget_get_surface(x) \
+	gtk_native_get_surface(gtk_widget_get_native(GTK_WIDGET(x)))
+
 G_BEGIN_DECLS
 
 typedef enum TrackType TrackType;
 
 typedef struct _CelluloidPlaylistEntry CelluloidPlaylistEntry;
 typedef struct _CelluloidMetadataEntry CelluloidMetadataEntry;
+typedef struct CelluloidChapter CelluloidChapter;
 typedef struct CelluloidTrack CelluloidTrack;
 typedef struct CelluloidDisc CelluloidDisc;
 
@@ -55,6 +59,12 @@ struct _CelluloidMetadataEntry
 	gchar *value;
 };
 
+struct CelluloidChapter
+{
+	gchar *title;
+	gdouble time;
+};
+
 struct CelluloidTrack
 {
 	TrackType type;
@@ -81,6 +91,12 @@ celluloid_metadata_entry_new(const gchar *key, const gchar *value);
 void
 celluloid_metadata_entry_free(CelluloidMetadataEntry *entry);
 
+CelluloidChapter *
+celluloid_chapter_new(void);
+
+void
+celluloid_chapter_free(CelluloidChapter *entry);
+
 CelluloidTrack *
 celluloid_track_new(void);
 
@@ -114,12 +130,12 @@ extension_matches(const gchar *filename, const gchar **extensions);
 gboolean
 g_source_clear(guint *tag);
 
-void *
-gslist_to_array(GSList *slist);
-
 gchar *
 strnjoinv(const gchar *separator, const gchar **str_array, gsize count);
 
+gchar *
+sanitize_utf8(const gchar *str, const gboolean label);
+
 char *
 format_time(gint seconds, gboolean show_hour);
 
