@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2020 gnome-mpv
+ * Copyright (c) 2017-2023 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -21,9 +21,11 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
+#ifdef G_OS_UNIX
 #include <glib-unix.h>
+#endif
 #include <locale.h>
-
+#include <adwaita.h>
 #include "celluloid-controller-private.h"
 #include "celluloid-controller.h"
 #include "celluloid-controller-actions.h"
@@ -64,9 +66,6 @@ boolean_to_loop(	GBinding *binding,
 static void
 mpris_enable_handler(GSettings *settings, gchar *key, gpointer data);
 
-static void
-media_keys_enable_handler(GSettings *settings, gchar *key, gpointer data);
-
 static void
 view_ready_handler(CelluloidView *view, gpointer data);
 
@@ -79,6 +78,9 @@ preferences_updated_handler(CelluloidView *view, gpointer data);
 static void
 audio_track_load_handler(CelluloidView *view, const gchar *uri, gpointer data);
 
+static void
+video_track_load_handler(CelluloidView *view, const gchar *uri, gpointer data);
+
 static void
 subtitle_track_load_handler(	CelluloidView *view,
 				const gchar *uri,
@@ -86,19 +88,22 @@ subtitle_track_load_handler(	CelluloidView *view,
 
 static void
 file_open_handler(	CelluloidView *view,
-			const gchar **uri_list,
+			GListModel *files,
 			gboolean append,
 			gpointer data );
 
 static void
-grab_handler(CelluloidView *view, gboolean was_grabbed, gpointer data);
+is_active_handler(GObject *gobject, GParamSpec *pspec, gpointer data);
 
 static gboolean
-delete_handler(CelluloidView *view, GdkEvent *event, gpointer data);
+close_request_handler(CelluloidView *view, gpointer data);
 
 static void
 playlist_item_activated_handler(CelluloidView *view, gint pos, gpointer data);
 
+static void
+playlist_item_inserted_handler(CelluloidView *view, gint pos, gpointer data);
+
 static void
 playlist_item_deleted_handler(CelluloidView *view, gint pos, gpointer data);
 
@@ -112,6 +117,9 @@ static void
 set_use_skip_button_for_playlist(	CelluloidController *controller,
 					gboolean value);
 
+static void
+set_dark_theme_enable(CelluloidController *controller, gboolean value);
+
 static void
 connect_signals(CelluloidController *controller);
 
@@ -139,11 +147,14 @@ window_scale_handler(GObject *object, GParamSpec *pspec, gpointer data);
 static void
 model_ready_handler(GObject *object, GParamSpec *pspec, gpointer data);
 
+static void
+playlist_replaced_handler(CelluloidModel *model, gpointer data);
+
 static void
 frame_ready_handler(CelluloidModel *model, gpointer data);
 
 static void
-metadata_update_handler(CelluloidModel *model, gint64 pos, gpointer data);
+metadata_cache_update_handler(CelluloidModel *model, gint64 pos, gpointer data);
 
 static void
 window_resize_handler(	CelluloidModel *model,
@@ -201,6 +212,9 @@ next_button_handler(GtkButton *button, gpointer data);
 static void
 previous_button_handler(GtkButton *button, gpointer data);
 
+static void
+playlist_button_handler(GtkButton *button, gpointer data);
+
 static void
 fullscreen_button_handler(GtkButton *button, gpointer data);
 
@@ -243,27 +257,27 @@ constructed(GObject *object)
 	celluloid_controller_input_connect_signals(controller);
 	update_extra_mpv_options(controller);
 
-	gtk_widget_show_all(GTK_WIDGET(window));
+	// If the window is already realized at this point (happens on X11), the
+	// ready signal will never fire. Because of this, we need to manually
+	// call view_ready_handler() to ensure that the model gets initialized.
+	if(gtk_widget_get_realized(GTK_WIDGET(window)))
+	{
+		view_ready_handler(controller->view, controller);
+	}
+
+	gtk_widget_add_controller(GTK_WIDGET(window), controller->key_controller);
+	gtk_widget_show(GTK_WIDGET(window));
 
 	g_signal_connect(	controller->settings,
 				"changed::mpris-enable",
 				G_CALLBACK(mpris_enable_handler),
 				controller );
-	g_signal_connect(	controller->settings,
-				"changed::media-keys-enable",
-				G_CALLBACK(media_keys_enable_handler),
-				controller );
 
 	if(g_settings_get_boolean(controller->settings, "mpris-enable"))
 	{
 		controller->mpris = celluloid_mpris_new(controller);
 	}
 
-	if(g_settings_get_boolean(controller->settings, "media-keys-enable"))
-	{
-		controller->media_keys = celluloid_media_keys_new(controller);
-	}
-
 	G_OBJECT_CLASS(celluloid_controller_parent_class)->constructed(object);
 }
 
@@ -295,6 +309,11 @@ set_property(	GObject *object,
 			(self, self->use_skip_buttons_for_playlist);
 		break;
 
+		case PROP_DARK_THEME_ENABLE:
+		self->dark_theme_enable = g_value_get_boolean(value);
+		set_dark_theme_enable(self, self->dark_theme_enable);
+		break;
+
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
 		break;
@@ -327,6 +346,10 @@ get_property(	GObject *object,
 		g_value_set_boolean(value, self->use_skip_buttons_for_playlist);
 		break;
 
+		case PROP_DARK_THEME_ENABLE:
+		g_value_set_boolean(value, self->dark_theme_enable);
+		break;
+
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
 		break;
@@ -340,7 +363,6 @@ dispose(GObject *object)
 
 	g_clear_object(&controller->settings);
 	g_clear_object(&controller->mpris);
-	g_clear_object(&controller->media_keys);
 
 	g_source_clear(&controller->update_seekbar_id);
 	g_source_clear(&controller->resize_timeout_tag);
@@ -349,7 +371,12 @@ dispose(GObject *object)
 	{
 		celluloid_view_make_gl_context_current(controller->view);
 		g_clear_object(&controller->model);
-		gtk_widget_destroy(GTK_WIDGET(controller->view));
+
+		// As of GTK 4.6.7, if the view is freed by calling
+		// g_object_unref(), it will never be removed from the list of
+		// toplevels. gtk_window_destroy() is used here instead to
+		// ensure that it's removed.
+		gtk_window_destroy(GTK_WINDOW(controller->view));
 		controller->view = NULL;
 	}
 
@@ -399,32 +426,23 @@ mpris_enable_handler(GSettings *settings, gchar *key, gpointer data)
 	}
 }
 
-static void
-media_keys_enable_handler(GSettings *settings, gchar *key, gpointer data)
-{
-	CelluloidController *controller = data;
-
-	if(!controller->media_keys && g_settings_get_boolean(settings, key))
-	{
-		controller->media_keys = celluloid_media_keys_new(controller);
-	}
-	else if(controller->media_keys)
-	{
-		g_clear_object(&controller->media_keys);
-	}
-}
-
 static void
 view_ready_handler(CelluloidView *view, gpointer data)
 {
 	CelluloidController *controller = CELLULOID_CONTROLLER(data);
 	CelluloidModel *model = controller->model;
+	gboolean maximized = FALSE;
 
 	celluloid_player_options_init
 		(	CELLULOID_PLAYER(controller->model),
 			CELLULOID_MAIN_WINDOW(controller->view) );
+	celluloid_model_initialize
+		(model);
 
-	celluloid_model_initialize(model);
+	g_object_get
+		(view, "maximized", &maximized, NULL);
+	celluloid_mpv_set_property_flag
+		(CELLULOID_MPV(model), "window-maximized", maximized);
 }
 
 static void
@@ -459,6 +477,12 @@ audio_track_load_handler(CelluloidView *view, const gchar *uri, gpointer data)
 	celluloid_model_load_audio_track(CELLULOID_CONTROLLER(data)->model, uri);
 }
 
+static void
+video_track_load_handler(CelluloidView *view, const gchar *uri, gpointer data)
+{
+	celluloid_model_load_video_track(CELLULOID_CONTROLLER(data)->model, uri);
+}
+
 static void
 subtitle_track_load_handler(	CelluloidView *view,
 				const gchar *uri,
@@ -470,30 +494,41 @@ subtitle_track_load_handler(	CelluloidView *view,
 
 static void
 file_open_handler(	CelluloidView *view,
-			const gchar **uri_list,
+			GListModel *files,
 			gboolean append,
 			gpointer data )
 {
-	for(const gchar **iter = uri_list; iter && *iter; iter++)
+	CelluloidModel *model = CELLULOID_CONTROLLER(data)->model;
+	guint files_count = g_list_model_get_n_items(files);
+
+	for(guint i = 0; i < files_count; i++)
 	{
-		celluloid_model_load_file
-			(	CELLULOID_CONTROLLER(data)->model,
-				*iter,
-				append || iter != uri_list );
+		GFile *file = g_list_model_get_item(files, i);
+		gchar *uri = g_file_get_path(file) ?: g_file_get_uri(file);
+
+		celluloid_model_load_file(model, uri, append || i > 0);
+
+		g_free(uri);
 	}
 }
 
 static void
-grab_handler(CelluloidView *view, gboolean was_grabbed, gpointer data)
+is_active_handler(GObject *gobject, GParamSpec *pspec, gpointer data)
 {
-	if(!was_grabbed)
+	CelluloidController *controller = CELLULOID_CONTROLLER(data);
+	CelluloidView *view = CELLULOID_VIEW(gobject);
+	gboolean is_active = TRUE;
+
+	g_object_get(view, "is-active", &is_active, NULL);
+
+	if(!is_active)
 	{
-		celluloid_model_reset_keys(CELLULOID_CONTROLLER(data)->model);
+		celluloid_model_reset_keys(controller->model);
 	}
 }
 
 static gboolean
-delete_handler(CelluloidView *view, GdkEvent *event, gpointer data)
+close_request_handler(CelluloidView *view, gpointer data)
 {
 	g_signal_emit_by_name(data, "shutdown");
 
@@ -504,21 +539,49 @@ static void
 playlist_item_activated_handler(CelluloidView *view, gint pos, gpointer data)
 {
 	CelluloidController *controller = CELLULOID_CONTROLLER(data);
+	CelluloidModel *model = controller->model;
 	gboolean idle_active = FALSE;
+	gint64 playlist_pos = -1;
 
-	g_object_get(controller->model, "idle-active", &idle_active, NULL);
-	celluloid_model_play(controller->model);
+	g_object_get(	model,
+			"idle-active", &idle_active,
+			"playlist-pos", &playlist_pos,
+			NULL );
+	celluloid_model_play(model);
 
 	if(idle_active)
 	{
 		controller->target_playlist_pos = pos;
 	}
+	else if(pos == playlist_pos)
+	{
+		celluloid_model_seek(model, 0.0);
+	}
 	else
 	{
-		celluloid_model_set_playlist_position(controller->model, pos);
+		celluloid_model_set_playlist_position(model, pos);
 	}
 }
 
+static void
+playlist_item_inserted_handler(CelluloidView *view, gint pos, gpointer data)
+{
+	CelluloidModel *model = CELLULOID_CONTROLLER(data)->model;
+	CelluloidMainWindow *window = CELLULOID_MAIN_WINDOW(view);
+	CelluloidPlaylistWidget *playlist = celluloid_main_window_get_playlist(window);
+	GPtrArray *contents = celluloid_playlist_widget_get_contents(playlist);
+
+	g_assert(contents->len > 0);
+	CelluloidPlaylistEntry *entry = g_ptr_array_index(contents, pos);
+
+	if((guint)pos != contents->len - 1)
+	{
+		g_warning("Playlist item inserted at non-last position. This is not yet supported. Appending to the playlist instead.");
+	}
+
+	celluloid_model_load_file(model, entry->filename, TRUE);
+}
+
 static void
 playlist_item_deleted_handler(CelluloidView *view, gint pos, gpointer data)
 {
@@ -558,6 +621,24 @@ set_use_skip_button_for_playlist(	CelluloidController *controller,
 				NULL );
 }
 
+static void
+set_dark_theme_enable(	CelluloidController *controller,
+					gboolean value )
+{
+	if(controller->dark_theme_enable)
+	{
+		adw_style_manager_set_color_scheme
+			(	adw_style_manager_get_default(),
+				ADW_COLOR_SCHEME_PREFER_DARK );
+	}
+	else
+	{
+		adw_style_manager_set_color_scheme
+			(	adw_style_manager_get_default(),
+				ADW_COLOR_SCHEME_DEFAULT );
+	}
+}
+
 static void
 connect_signals(CelluloidController *controller)
 {
@@ -568,7 +649,7 @@ connect_signals(CelluloidController *controller)
 				controller->view, "border",
 				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "fullscreen",
-				controller->view, "fullscreen",
+				controller->view, "fullscreened",
 				G_BINDING_BIDIRECTIONAL );
 	g_object_bind_property(	controller->model, "window-maximized",
 				controller->view, "maximized",
@@ -576,8 +657,11 @@ connect_signals(CelluloidController *controller)
 	g_object_bind_property(	controller->model, "pause",
 				controller->view, "pause",
 				G_BINDING_DEFAULT );
+	g_object_bind_property(	controller->model, "idle-active",
+				controller->view, "idle-active",
+				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "media-title",
-				controller->view, "title",
+				controller->view, "media-title",
 				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "volume",
 				controller->view, "volume",
@@ -591,15 +675,15 @@ connect_signals(CelluloidController *controller)
 	g_object_bind_property(	controller->model, "playlist-pos",
 				controller->view, "playlist-pos",
 				G_BINDING_DEFAULT );
+	g_object_bind_property(	controller->model, "chapter-list",
+				controller->view, "chapter-list",
+				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "track-list",
 				controller->view, "track-list",
 				G_BINDING_DEFAULT );
 	g_object_bind_property(	controller->model, "disc-list",
 				controller->view, "disc-list",
 				G_BINDING_DEFAULT );
-	g_object_bind_property(	controller->view, "display-fps",
-				controller->model, "display-fps",
-				G_BINDING_DEFAULT|G_BINDING_SYNC_CREATE );
 	g_object_bind_property_full(	controller->view, "loop",
 					controller->model, "loop-playlist",
 					G_BINDING_BIDIRECTIONAL|
@@ -608,6 +692,9 @@ connect_signals(CelluloidController *controller)
 					loop_to_boolean,
 					NULL,
 					NULL );
+	g_object_bind_property(	controller->view, "shuffle",
+				controller->model, "shuffle",
+				G_BINDING_BIDIRECTIONAL|G_BINDING_SYNC_CREATE );
 
 	g_signal_connect(	controller->model,
 				"notify::ready",
@@ -629,13 +716,17 @@ connect_signals(CelluloidController *controller)
 				"notify::window-scale",
 				G_CALLBACK(window_scale_handler),
 				controller );
+	g_signal_connect(	controller->model,
+				"playlist-replaced",
+				G_CALLBACK(playlist_replaced_handler),
+				controller );
 	g_signal_connect(	controller->model,
 				"frame-ready",
 				G_CALLBACK(frame_ready_handler),
 				controller );
 	g_signal_connect(	controller->model,
-				"metadata-update",
-				G_CALLBACK(metadata_update_handler),
+				"metadata-cache-update",
+				G_CALLBACK(metadata_cache_update_handler),
 				controller );
 	g_signal_connect(	controller->model,
 				"window-resize",
@@ -694,6 +785,10 @@ connect_signals(CelluloidController *controller)
 				"button-clicked::previous",
 				G_CALLBACK(previous_button_handler),
 				controller );
+	g_signal_connect(	controller->view,
+				"button-clicked::playlist",
+				G_CALLBACK(playlist_button_handler),
+				controller );
 	g_signal_connect(	controller->view,
 				"button-clicked::fullscreen",
 				G_CALLBACK(fullscreen_button_handler),
@@ -719,6 +814,10 @@ connect_signals(CelluloidController *controller)
 				"audio-track-load",
 				G_CALLBACK(audio_track_load_handler),
 				controller );
+	g_signal_connect(	controller->view,
+				"video-track-load",
+				G_CALLBACK(video_track_load_handler),
+				controller );
 	g_signal_connect(	controller->view,
 				"subtitle-track-load",
 				G_CALLBACK(subtitle_track_load_handler),
@@ -728,17 +827,21 @@ connect_signals(CelluloidController *controller)
 				G_CALLBACK(file_open_handler),
 				controller );
 	g_signal_connect(	controller->view,
-				"grab-notify",
-				G_CALLBACK(grab_handler),
+				"notify::is-active",
+				G_CALLBACK(is_active_handler),
 				controller );
 	g_signal_connect(	controller->view,
-				"delete-event",
-				G_CALLBACK(delete_handler),
+				"close-request",
+				G_CALLBACK(close_request_handler),
 				controller );
 	g_signal_connect(	controller->view,
 				"playlist-item-activated",
 				G_CALLBACK(playlist_item_activated_handler),
 				controller );
+	g_signal_connect(	controller->view,
+				"playlist-item-inserted",
+				G_CALLBACK(playlist_item_inserted_handler),
+				controller );
 	g_signal_connect(	controller->view,
 				"playlist-item-deleted",
 				G_CALLBACK(playlist_item_deleted_handler),
@@ -851,7 +954,7 @@ window_scale_handler(GObject *object, GParamSpec *pspec, gpointer data)
 	window_scale = MIN(	width/(gdouble)video_width,
 				height/(gdouble)video_height );
 
-	if(ABS(window_scale-new_window_scale) > 0.0001)
+	if(window_scale > 0.0 && ABS(window_scale-new_window_scale) > 0.0001)
 	{
 		celluloid_controller_autofit(data, new_window_scale);
 	}
@@ -872,9 +975,6 @@ model_ready_handler(GObject *object, GParamSpec *pspec, gpointer data)
 		use_opengl_cb =	celluloid_model_get_use_opengl_cb
 				(controller->model);
 
-		celluloid_view_set_use_opengl_cb
-			(controller->view, use_opengl_cb);
-
 		if(use_opengl_cb)
 		{
 			celluloid_view_make_gl_context_current(controller->view);
@@ -892,6 +992,19 @@ model_ready_handler(GObject *object, GParamSpec *pspec, gpointer data)
 	g_object_notify(data, "ready");
 }
 
+static void
+playlist_replaced_handler(CelluloidModel *model, gpointer data)
+{
+	GSettings *settings = g_settings_new(CONFIG_ROOT);
+
+	if(g_settings_get_boolean(settings, "present-window-on-file-open"))
+	{
+		celluloid_view_present(CELLULOID_CONTROLLER(data)->view);
+	}
+
+	g_object_unref(settings);
+}
+
 static void
 frame_ready_handler(CelluloidModel *model, gpointer data)
 {
@@ -899,7 +1012,7 @@ frame_ready_handler(CelluloidModel *model, gpointer data)
 }
 
 static void
-metadata_update_handler(CelluloidModel *model, gint64 pos, gpointer data)
+metadata_cache_update_handler(CelluloidModel *model, gint64 pos, gpointer data)
 {
 	CelluloidView *view = CELLULOID_CONTROLLER(data)->view;
 	GPtrArray *playlist = NULL;
@@ -970,12 +1083,8 @@ message_handler(CelluloidMpv *mpv, const gchar *message, gpointer data)
 static void
 error_handler(CelluloidMpv *mpv, const gchar *message, gpointer data)
 {
-	celluloid_view_show_message_dialog
-		(	CELLULOID_CONTROLLER(data)->view,
-			GTK_MESSAGE_ERROR,
-			_("Error"),
-			NULL,
-			message );
+	celluloid_view_show_message_toast
+		(CELLULOID_CONTROLLER(data)->view, message);
 }
 
 static void
@@ -1048,18 +1157,12 @@ fullscreen_handler(GObject *object, GParamSpec *pspec, gpointer data)
 	CelluloidMainWindow *window = celluloid_view_get_main_window(view);
 	GActionMap *map = G_ACTION_MAP(window);
 	GAction *toggle_playlist = NULL;
-	GAction *toggle_controls = NULL;
-	gboolean fullscreen = FALSE;
-
-	toggle_playlist = g_action_map_lookup_action(map, "toggle-playlist");
-	toggle_controls = g_action_map_lookup_action(map, "toggle-controls");
-
-	g_object_get(view, "fullscreen", &fullscreen, NULL);
+	const gboolean fullscreen = gtk_window_is_fullscreen(GTK_WINDOW(view));
 
+	toggle_playlist =
+		g_action_map_lookup_action(map, "toggle-playlist");
 	g_simple_action_set_enabled
 		(G_SIMPLE_ACTION(toggle_playlist), !fullscreen);
-	g_simple_action_set_enabled
-		(G_SIMPLE_ACTION(toggle_controls), !fullscreen);
 }
 
 static void
@@ -1142,14 +1245,24 @@ previous_button_handler(GtkButton *button, gpointer data)
 	}
 }
 
+static void
+playlist_button_handler(GtkButton *button, gpointer data)
+{
+	CelluloidView *view =
+		CELLULOID_CONTROLLER(data)->view;
+	const gboolean playlist_visible =
+		celluloid_view_get_playlist_visible(view);
+
+	celluloid_view_set_playlist_visible(view, !playlist_visible);
+}
+
 static void
 fullscreen_button_handler(GtkButton *button, gpointer data)
 {
 	CelluloidView *view = CELLULOID_CONTROLLER(data)->view;
-	gboolean fullscreen = FALSE;
+	const gboolean fullscreen = gtk_window_is_fullscreen(GTK_WINDOW(view));
 
-	g_object_get(view, "fullscreen", &fullscreen, NULL);
-	g_object_set(view, "fullscreen", !fullscreen, NULL);
+	g_object_set(view, "fullscreened", !fullscreen, NULL);
 }
 
 static void
@@ -1200,6 +1313,14 @@ celluloid_controller_class_init(CelluloidControllerClass *klass)
 			G_PARAM_READWRITE );
 	g_object_class_install_property(obj_class, PROP_USE_SKIP_BUTTONS_FOR_PLAYLIST, pspec);
 
+	pspec = g_param_spec_boolean
+		(	"dark-theme-enable",
+			"Enable dark theme",
+			"Whether or not to enable dark theme",
+			FALSE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(obj_class, PROP_DARK_THEME_ENABLE, pspec);
+
 	g_signal_new(	"shutdown",
 			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
@@ -1214,6 +1335,11 @@ celluloid_controller_class_init(CelluloidControllerClass *klass)
 static void
 celluloid_controller_init(CelluloidController *controller)
 {
+	controller->key_controller = gtk_event_controller_key_new();
+
+	gtk_event_controller_set_propagation_phase
+		(controller->key_controller, GTK_PHASE_CAPTURE);
+
 	controller->app = NULL;
 	controller->model = NULL;
 	controller->view = NULL;
@@ -1224,7 +1350,6 @@ celluloid_controller_init(CelluloidController *controller)
 	controller->resize_timeout_tag = 0;
 	controller->skip_buttons_binding = NULL;
 	controller->settings = g_settings_new(CONFIG_ROOT);
-	controller->media_keys = NULL;
 	controller->mpris = NULL;
 }
 
@@ -1274,11 +1399,12 @@ celluloid_controller_autofit(	CelluloidController *controller,
 	{
 		gint new_width = (gint)(multiplier*(gdouble)width);
 		gint new_height = (gint)(multiplier*(gdouble)height);
+		gint scale = celluloid_view_get_scale_factor(controller->view);
 
 		g_debug("Resizing window to %dx%d", new_width, new_height);
 		celluloid_view_resize_video_area(	controller->view,
-							new_width,
-							new_height );
+							new_width/scale,
+							new_height/scale );
 	}
 
 	g_free(vid);
