@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2022 gnome-mpv
+ * Copyright (c) 2017-2023 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -512,8 +512,7 @@ apply_default_options(CelluloidMpv *mpv)
 		const gchar *name;
 		const gchar *value;
 	}
-	options[] = {	{"vo", "opengl,vdpau,vaapi,xv,x11,libmpv,"},
-			{"osd-level", "1"},
+	options[] = {	{"osd-level", "1"},
 			{"softvol", "yes"},
 			{"force-window", "immediate"},
 			{"input-default-bindings", "yes"},
@@ -531,6 +530,7 @@ apply_default_options(CelluloidMpv *mpv)
 			{"softvol-max", "100"},
 			{"config-dir", config_dir},
 			{"watch-later-directory", watch_dir},
+			{"screenshot-format", "png"},
 			{"screenshot-directory", screenshot_dir},
 			{"screenshot-template", "%f-%P"},
 			{NULL, NULL} };
@@ -788,16 +788,26 @@ load_config_file(CelluloidMpv *mpv)
 
 	if(g_settings_get_boolean(settings, "mpv-config-enable"))
 	{
-		gchar *mpv_conf =	g_settings_get_string
-					(settings, "mpv-config-file");
+		gchar *mpv_conf =
+			g_settings_get_string(settings, "mpv-config-file");
 
 		GFile *file = g_file_new_for_uri(mpv_conf);
 		gchar *path = g_file_get_path(file);
 
-		g_info("Loading config file: %s", path);
-		celluloid_mpv_load_config_file(mpv, path);
+		g_info("Loading mpv config file: %s", mpv_conf);
+
+		if(path)
+		{
+			g_debug("mpv config file path: %s", path);
+			celluloid_mpv_load_config_file(mpv, path);
+
+			g_free(path);
+		}
+		else
+		{
+			g_warning("Failed to load mpv config file");
+		}
 
-		g_free(path);
 		g_object_unref(file);
 		g_free(mpv_conf);
 	}
