@@ -22,6 +22,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n.h>
+#include <adwaita.h>
 
 #include "celluloid-preferences-dialog.h"
 #include "celluloid-file-chooser-button.h"
@@ -34,23 +35,20 @@ typedef enum PreferencesDialogItemType PreferencesDialogItemType;
 
 struct _CelluloidPreferencesDialog
 {
-	GtkDialog parent_instance;
+	AdwPreferencesWindow parent_instance;
 	GSettings *settings;
-	GtkWidget *notebook;
 };
 
 struct _CelluloidPreferencesDialogClass
 {
-	GtkDialogClass parent_class;
+	AdwPreferencesWindowClass parent_class;
 };
 
 enum PreferencesDialogItemType
 {
 	ITEM_TYPE_INVALID,
-	ITEM_TYPE_GROUP,
-	ITEM_TYPE_CHECK_BOX,
+	ITEM_TYPE_SWITCH,
 	ITEM_TYPE_FILE_CHOOSER,
-	ITEM_TYPE_LABEL,
 	ITEM_TYPE_TEXT_BOX
 };
 
@@ -61,7 +59,7 @@ struct PreferencesDialogItem
 	PreferencesDialogItemType type;
 };
 
-G_DEFINE_TYPE(CelluloidPreferencesDialog, celluloid_preferences_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE(CelluloidPreferencesDialog, celluloid_preferences_dialog, ADW_TYPE_PREFERENCES_WINDOW)
 
 static void
 file_set_handler(CelluloidFileChooserButton *button, gpointer data)
@@ -78,40 +76,11 @@ file_set_handler(CelluloidFileChooserButton *button, gpointer data)
 	g_object_unref(file);
 }
 
-static void
-response_handler(GtkDialog *dialog, gint response_id)
+static gboolean
+save_settings(AdwPreferencesWindow *dialog)
 {
 	CelluloidPreferencesDialog *dlg = CELLULOID_PREFERENCES_DIALOG(dialog);
-
-	if(response_id == GTK_RESPONSE_ACCEPT)
-	{
-		g_settings_apply(dlg->settings);
-	}
-	else
-	{
-		g_settings_revert(dlg->settings);
-	}
-}
-
-static gboolean
-key_pressed_handler(	GtkEventControllerKey *controller,
-			guint keyval,
-			guint keycode,
-			GdkModifierType state,
-			gpointer data )
-{
-	const guint mod_mask =	GDK_MODIFIER_MASK
-				&~(GDK_SHIFT_MASK
-				|GDK_LOCK_MASK
-				|GDK_ALT_MASK
-				|GDK_SUPER_MASK
-				|GDK_HYPER_MASK
-				|GDK_META_MASK);
-
-	if((state&mod_mask) == 0 && keyval == GDK_KEY_Return)
-	{
-		gtk_dialog_response(GTK_DIALOG(data), GTK_RESPONSE_ACCEPT);
-	}
+	g_settings_apply(dlg->settings);
 
 	return FALSE;
 }
@@ -123,18 +92,23 @@ free_signal_data(gpointer data, GClosure *closure)
 }
 
 static GtkWidget *
-build_page(const PreferencesDialogItem *items, GSettings *settings)
+build_page(	const PreferencesDialogItem *items,
+		GSettings *settings,
+		const char *title,
+		const char *icon_name )
 {
-	GtkWidget *grid = gtk_grid_new();
+	GtkWidget *pref_page = adw_preferences_page_new();
+	adw_preferences_page_set_title
+		(ADW_PREFERENCES_PAGE(pref_page), title);
+	adw_preferences_page_set_icon_name
+		(ADW_PREFERENCES_PAGE(pref_page), icon_name);
+
+	GtkWidget *pref_group = adw_preferences_group_new();
+	adw_preferences_page_add
+		(	ADW_PREFERENCES_PAGE(pref_page),
+			ADW_PREFERENCES_GROUP(pref_group) );
+
 	GSettingsSchema *schema = NULL;
-
-	gtk_widget_set_margin_end(grid, 12);
-	gtk_widget_set_margin_top(grid, 12);
-	gtk_widget_set_margin_bottom(grid, 12);
-
-	gtk_grid_set_row_spacing(GTK_GRID(grid), 6);
-	gtk_grid_set_column_spacing(GTK_GRID(grid), 12);
-
 	g_object_get(settings, "settings-schema", &schema, NULL);
 
 	for(gint i = 0; items[i].type != ITEM_TYPE_INVALID; i++)
@@ -150,66 +124,74 @@ build_page(const PreferencesDialogItem *items, GSettings *settings)
 			NULL;
 		const gchar *label = items[i].label ?: summary;
 		const PreferencesDialogItemType type = items[i].type;
+
 		GtkWidget *widget = NULL;
-		gboolean separate_label = FALSE;
-		gint width = 1;
-		gint xpos = 0;
 
-		if(type == ITEM_TYPE_GROUP)
+		if(type == ITEM_TYPE_SWITCH)
 		{
-			widget = gtk_label_new(label);
-			width = 2;
+			GtkWidget *pref_switch;
 
-			gtk_label_set_use_markup(GTK_LABEL(widget), TRUE);
-			gtk_widget_set_halign(widget, GTK_ALIGN_START);
-			gtk_widget_set_margin_top(widget, 12);
-		}
-		else if(type == ITEM_TYPE_CHECK_BOX)
-		{
-			widget = gtk_check_button_new_with_label(label);
-			width = 2;
+			widget = adw_action_row_new();
+			adw_preferences_row_set_title
+				(ADW_PREFERENCES_ROW(widget), label);
+
+			pref_switch = gtk_switch_new();
+			gtk_widget_set_valign
+				(pref_switch, GTK_ALIGN_CENTER);
+			adw_action_row_add_suffix
+				(ADW_ACTION_ROW(widget), pref_switch);
+			adw_action_row_set_activatable_widget
+				(ADW_ACTION_ROW(widget), pref_switch);
 
 			g_settings_bind(	settings,
 						key,
-						widget,
+						pref_switch,
 						"active",
 						G_SETTINGS_BIND_DEFAULT );
 		}
 		else if(type == ITEM_TYPE_FILE_CHOOSER)
 		{
-			CelluloidFileChooserButton *button;
+			GtkWidget *button;
 			GtkFileFilter *filter;
 			gchar *uri;
 			GFile *file;
 
-			widget =	celluloid_file_chooser_button_new
-					(NULL, GTK_FILE_CHOOSER_ACTION_OPEN);
+			widget = adw_action_row_new();
+			adw_preferences_row_set_title
+				(ADW_PREFERENCES_ROW(widget), label);
+
+			button = celluloid_file_chooser_button_new
+				(	NULL,
+					GTK_FILE_CHOOSER_ACTION_OPEN );
+			gtk_widget_set_valign
+				(button, GTK_ALIGN_CENTER);
+			adw_action_row_add_suffix
+				(ADW_ACTION_ROW(widget), button);
+			adw_action_row_set_activatable_widget
+				(ADW_ACTION_ROW(widget), button);
 
-			button = CELLULOID_FILE_CHOOSER_BUTTON(widget);
 			filter = gtk_file_filter_new();
 			uri = g_settings_get_string(settings, key);
 			file = g_file_new_for_uri(uri);
 
-			separate_label = TRUE;
-			xpos = 1;
-
-			gtk_widget_set_hexpand(widget, TRUE);
-			gtk_widget_set_size_request(widget, 100, -1);
-
-			gtk_file_filter_add_mime_type(filter, "text/plain");
-			celluloid_file_chooser_button_set_filter(button, filter);
+			gtk_file_filter_add_mime_type
+				(filter, "text/plain");
+			celluloid_file_chooser_button_set_filter
+				(CELLULOID_FILE_CHOOSER_BUTTON(button), filter);
 
 			if(g_file_query_exists(file, NULL))
 			{
 				celluloid_file_chooser_button_set_file
-					(button, file, NULL);
+					(	CELLULOID_FILE_CHOOSER_BUTTON(button),
+						 file,
+						 NULL );
 			}
 
 			/* For simplicity, changes made to the GSettings
 			 * database externally won't be reflected immediately
 			 * for this type of widget.
 			 */
-			g_signal_connect_data(	widget,
+			g_signal_connect_data(	button,
 						"file-set",
 						G_CALLBACK(file_set_handler),
 						g_strdup(key),
@@ -220,11 +202,12 @@ build_page(const PreferencesDialogItem *items, GSettings *settings)
 		}
 		else if(type == ITEM_TYPE_TEXT_BOX)
 		{
-			widget = gtk_entry_new();
-			separate_label = TRUE;
-			xpos = 1;
 
-			gtk_widget_set_hexpand(widget, TRUE);
+			widget = adw_entry_row_new();
+			adw_preferences_row_set_title
+				(ADW_PREFERENCES_ROW(widget), label);
+			gtk_widget_set_hexpand
+				(widget, TRUE);
 
 			g_settings_bind(	settings,
 						key,
@@ -232,79 +215,17 @@ build_page(const PreferencesDialogItem *items, GSettings *settings)
 						"text",
 						G_SETTINGS_BIND_DEFAULT );
 		}
-		else if(type == ITEM_TYPE_LABEL)
-		{
-			widget = gtk_label_new(label);
 
-			gtk_widget_set_halign(widget, GTK_ALIGN_START);
-		}
-
-		g_assert(widget);
-		g_assert(xpos == 0 || xpos == 1);
-
-		if(i == 0)
-		{
-			gtk_widget_set_margin_top(widget, 0);
-		}
-
-		if(type != ITEM_TYPE_GROUP)
-		{
-			gtk_widget_set_margin_start(widget, 12);
-		}
-
-		/* Expand the widget to fill both columns if it usually needs a
-		 * separate label but none is provided.
-		 */
-		if(separate_label && label && !label[0])
-		{
-			width = 2;
-			xpos = 0;
-		}
-
-		gtk_grid_attach(GTK_GRID(grid), widget, xpos, i, width, 1);
-
-		if(separate_label && label && label[0])
-		{
-			GtkWidget *label_widget = gtk_label_new(label);
-
-			/* The grid should only have 2 columns, so the previous
-			 * widget connot be wider than 1 column if it needs a
-			 * separate label.
-			 */
-			g_assert(width == 1);
-
-			gtk_grid_attach(	GTK_GRID(grid),
-						label_widget,
-						1-xpos, i, 1, 1 );
-
-			gtk_widget_set_halign(label_widget, GTK_ALIGN_START);
-			gtk_widget_set_hexpand(label_widget, FALSE);
-			gtk_widget_set_margin_start(label_widget, 12);
-		}
+		adw_preferences_group_add
+			(ADW_PREFERENCES_GROUP(pref_group), widget);
 	}
 
-	return grid;
+	return pref_page;
 }
 
 static void
 preferences_dialog_constructed(GObject *obj)
 {
-	gboolean csd_enabled;
-
-	g_object_get(obj, "use-header-bar", &csd_enabled, NULL);
-
-	if(!csd_enabled)
-	{
-		GtkWidget *content_area;
-		GtkWidget *notebook;
-
-		content_area = gtk_dialog_get_content_area(GTK_DIALOG(obj));
-		notebook = CELLULOID_PREFERENCES_DIALOG(obj)->notebook;
-
-		gtk_widget_set_margin_bottom(content_area, 12);
-		gtk_widget_set_margin_bottom(notebook, 12);
-	}
-
 	G_OBJECT_CLASS(celluloid_preferences_dialog_parent_class)->constructed(obj);
 }
 
@@ -322,7 +243,6 @@ finalize(GObject *object)
 static void
 celluloid_preferences_dialog_class_init(CelluloidPreferencesDialogClass *klass)
 {
-	GTK_DIALOG_CLASS(klass)->response = response_handler;
 	G_OBJECT_CLASS(klass)->constructed = preferences_dialog_constructed;
 	G_OBJECT_CLASS(klass)->finalize = finalize;
 }
@@ -333,45 +253,45 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 	const PreferencesDialogItem interface_items[]
 		= {	{NULL,
 			"autofit-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"csd-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"dark-theme-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"always-use-floating-controls",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"draggable-video-area-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"always-show-title-buttons",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"present-window-on-file-open",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"always-autohide-cursor",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"use-skip-buttons-for-playlist",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"last-folder-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL, NULL, ITEM_TYPE_INVALID} };
 	const PreferencesDialogItem config_items[]
 		= {	{NULL,
 			"mpv-config-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{_("mpv configuration file:"),
 			"mpv-config-file",
 			ITEM_TYPE_FILE_CHOOSER},
 			{NULL,
 			"mpv-input-config-enable",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{_("mpv input configuration file:"),
 			"mpv-input-config-file",
 			ITEM_TYPE_FILE_CHOOSER},
@@ -379,96 +299,75 @@ celluloid_preferences_dialog_init(CelluloidPreferencesDialog *dlg)
 	const PreferencesDialogItem misc_items[]
 		= {	{NULL,
 			"always-open-new-window",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"always-append-to-playlist",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"ignore-playback-errors",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"prefetch-metadata",
-			ITEM_TYPE_CHECK_BOX},
+			ITEM_TYPE_SWITCH},
 			{NULL,
 			"mpris-enable",
-			ITEM_TYPE_CHECK_BOX},
-			{NULL,
-			"media-keys-enable",
-			ITEM_TYPE_CHECK_BOX},
-			{_("Extra mpv options:"),
-			NULL,
-			ITEM_TYPE_LABEL},
-			{"",
+			ITEM_TYPE_SWITCH},
+			{_("Extra mpv options"),
 			"mpv-options",
 			ITEM_TYPE_TEXT_BOX},
 			{NULL, NULL, ITEM_TYPE_INVALID} };
 
-	GtkWidget *content_area;
-
 	dlg->settings = g_settings_new(CONFIG_ROOT);
-	dlg->notebook = gtk_notebook_new();
-	content_area = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
-
 	g_settings_delay(dlg->settings);
 
-	gtk_box_append(GTK_BOX(content_area), dlg->notebook);
+	GtkWidget *page = NULL;
 
-	gtk_notebook_append_page(	GTK_NOTEBOOK(dlg->notebook),
-					build_page(interface_items, dlg->settings),
-					gtk_label_new(_("Interface")) );
-	gtk_notebook_append_page(	GTK_NOTEBOOK(dlg->notebook),
-					build_page(config_items, dlg->settings),
-					gtk_label_new(_("Config Files")) );
-	gtk_notebook_append_page(	GTK_NOTEBOOK(dlg->notebook),
-					build_page(misc_items, dlg->settings),
-					gtk_label_new(_("Miscellaneous")) );
-	gtk_notebook_append_page(	GTK_NOTEBOOK(dlg->notebook),
-					celluloid_plugins_manager_new(GTK_WINDOW(dlg)),
-					gtk_label_new(_("Plugins")) );
+	page = build_page
+		(	interface_items,
+			dlg->settings,
+			_("Interface"),
+			"preferences-desktop-appearance-symbolic" );
+	adw_preferences_window_add(	ADW_PREFERENCES_WINDOW(dlg),
+					ADW_PREFERENCES_PAGE(page));
 
-	gtk_dialog_add_buttons(	GTK_DIALOG(dlg),
-				_("_Cancel"),
-				GTK_RESPONSE_CANCEL,
-				_("_Save"),
-				GTK_RESPONSE_ACCEPT,
+	page = build_page
+		(	config_items,
+			dlg->settings,
+			_("Config Files"),
+			"document-properties-symbolic" );
+	adw_preferences_window_add(	ADW_PREFERENCES_WINDOW(dlg),
+					ADW_PREFERENCES_PAGE(page));
+
+	page = build_page
+		(	misc_items,
+			dlg->settings,
+			_("Miscellaneous"),
+			"preferences-other-symbolic" );
+	adw_preferences_window_add(	ADW_PREFERENCES_WINDOW(dlg),
+					ADW_PREFERENCES_PAGE(page));
+
+	page = GTK_WIDGET(celluloid_plugins_manager_new(GTK_WINDOW(dlg)));
+	adw_preferences_window_add(	ADW_PREFERENCES_WINDOW(dlg),
+					ADW_PREFERENCES_PAGE(page) );
+
+	g_signal_connect(	dlg,
+				"close-request",
+				G_CALLBACK(save_settings),
 				NULL );
-
-	gtk_dialog_set_default_response(GTK_DIALOG(dlg), GTK_RESPONSE_ACCEPT);
-
-	GtkEventController *key_controller = gtk_event_controller_key_new();
-
-	gtk_widget_add_controller(GTK_WIDGET(dlg), key_controller);
-
-	g_signal_connect(	key_controller,
-				"key-pressed",
-				G_CALLBACK(key_pressed_handler),
-				dlg );
 }
 
 GtkWidget *
 celluloid_preferences_dialog_new(GtkWindow *parent)
 {
 	GtkWidget *dlg;
-	GtkWidget *header_bar;
-	gboolean csd_enabled;
 
-	csd_enabled = celluloid_main_window_get_csd_enabled(CELLULOID_MAIN_WINDOW(parent));
 
 	dlg = g_object_new(	celluloid_preferences_dialog_get_type(),
 				"title", _("Preferences"),
 				"modal", TRUE,
 				"transient-for", parent,
-				"use-header-bar", csd_enabled,
 				NULL );
 
-	header_bar = gtk_dialog_get_header_bar(GTK_DIALOG(dlg));
-
-	if(header_bar)
-	{
-		gtk_header_bar_set_show_title_buttons
-			(GTK_HEADER_BAR(header_bar), FALSE);
-	}
-
 	gtk_widget_show(dlg);
 
 	return dlg;
