@@ -152,9 +152,8 @@ open_track_dialog_response_handler(	GtkDialog *dialog,
 					gint response_id,
 					gpointer data );
 
-static void
-preferences_dialog_response_handler(	GtkDialog *dialog,
-					gint response_id,
+static gboolean
+preferences_dialog_response_handler(	AdwPreferencesWindow *dialog,
 					gpointer data );
 
 static void
@@ -339,6 +338,8 @@ set_property(	GObject *object,
 		CELLULOID_MAIN_WINDOW(self);
 	CelluloidControlBox* control_box =
 		celluloid_main_window_get_control_box(wnd);
+	CelluloidVideoArea *area =
+		celluloid_main_window_get_video_area(wnd);
 
 	switch(property_id)
 	{
@@ -365,6 +366,11 @@ set_property(	GObject *object,
 		case PROP_IDLE_ACTIVE:
 		self->idle_active = g_value_get_boolean(value);
 		update_title(self);
+		if(!self->idle_active)
+		{
+			celluloid_video_area_set_initial_page_visible
+				(area, FALSE);
+		}
 		break;
 
 		case PROP_VOLUME:
@@ -915,40 +921,36 @@ open_track_dialog_response_handler(	GtkDialog *dialog,
 	celluloid_file_chooser_destroy(CELLULOID_FILE_CHOOSER(dialog));
 }
 
-static void
-preferences_dialog_response_handler(	GtkDialog *dialog,
-					gint response_id,
+static gboolean
+preferences_dialog_response_handler(	AdwPreferencesWindow *dialog,
 					gpointer data )
 {
-	if(response_id == GTK_RESPONSE_ACCEPT)
+	CelluloidMainWindow *wnd;
+	GSettings *settings;
+	gboolean csd_enable;
+
+	wnd = CELLULOID_MAIN_WINDOW(data);
+	settings = g_settings_new(CONFIG_ROOT);
+	csd_enable = g_settings_get_boolean(settings, "csd-enable");
+
+	if(celluloid_main_window_get_csd_enabled(wnd) != csd_enable)
 	{
-		CelluloidMainWindow *wnd;
-		GSettings *settings;
-		gboolean csd_enable;
-
-		wnd = CELLULOID_MAIN_WINDOW(data);
-		settings = g_settings_new(CONFIG_ROOT);
-		csd_enable = g_settings_get_boolean(settings, "csd-enable");
-
-		if(celluloid_main_window_get_csd_enabled(wnd) != csd_enable)
-		{
-			show_message_dialog(	CELLULOID_VIEW(data),
-						GTK_MESSAGE_INFO,
-						g_get_application_name(),
-						NULL,
-						_("Enabling or disabling "
-						"client-side decorations "
-						"requires restarting to "
-						"take effect.") );
-		}
-
-		gtk_widget_queue_draw(GTK_WIDGET(wnd));
-		g_signal_emit_by_name(data, "preferences-updated");
-
-		g_object_unref(settings);
+		show_message_dialog(	CELLULOID_VIEW(data),
+					GTK_MESSAGE_INFO,
+					g_get_application_name(),
+					NULL,
+					_("Enabling or disabling "
+					"client-side decorations "
+					"requires restarting to "
+					"take effect.") );
 	}
 
-	gtk_window_close(GTK_WINDOW(dialog));
+	gtk_widget_queue_draw(GTK_WIDGET(wnd));
+	g_signal_emit_by_name(data, "preferences-updated");
+
+	g_object_unref(settings);
+
+	return FALSE;
 }
 
 static void
@@ -1572,7 +1574,7 @@ celluloid_view_show_preferences_dialog(CelluloidView *view)
 	GtkWidget *dialog = celluloid_preferences_dialog_new(GTK_WINDOW(view));
 
 	g_signal_connect_after(	dialog,
-				"response",
+				"close-request",
 				G_CALLBACK(preferences_dialog_response_handler),
 				view );
 
@@ -1617,13 +1619,14 @@ celluloid_view_show_about_window (CelluloidView *view)
 }
 
 void
-celluloid_view_show_message_dialog(	CelluloidView *view,
-					GtkMessageType type,
-					const gchar *title,
-					const gchar *prefix,
-					const gchar *msg )
+celluloid_view_show_message_toast(CelluloidView *view, const gchar *msg)
 {
-	show_message_dialog(view, type, title, prefix, msg);
+	CelluloidMainWindow *wnd =
+		CELLULOID_MAIN_WINDOW(view);
+	CelluloidVideoArea *video_area =
+		celluloid_main_window_get_video_area(wnd);
+
+	celluloid_video_area_show_toast_message(video_area, msg);
 }
 
 void
@@ -1667,15 +1670,6 @@ celluloid_view_make_gl_context_current(CelluloidView *view)
 	gtk_gl_area_make_current(gl_area);
 }
 
-void
-celluloid_view_set_use_opengl_cb(CelluloidView *view, gboolean use_opengl_cb)
-{
-	CelluloidMainWindow *wnd = CELLULOID_MAIN_WINDOW(view);
-	CelluloidVideoArea *area = celluloid_main_window_get_video_area(wnd);
-
-	celluloid_video_area_set_use_opengl(area, use_opengl_cb);
-}
-
 gint
 celluloid_view_get_scale_factor(CelluloidView *view)
 {
